@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.c,v 1.229 2020/05/11 16:59:19 claudio Exp $ */
+/*	$OpenBSD: bgpd.c,v 1.234 2021/02/16 08:29:16 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -47,7 +47,9 @@ int		send_config(struct bgpd_config *);
 int		dispatch_imsg(struct imsgbuf *, int, struct bgpd_config *);
 int		control_setup(struct bgpd_config *);
 static void	getsockpair(int [2]);
-int		imsg_send_sockets(struct imsgbuf *, struct imsgbuf *);
+int		imsg_send_sockets(struct imsgbuf *, struct imsgbuf *,
+		    struct imsgbuf *);
+void		bgpd_rtr_connect(struct rtr_config *);
 
 int			 cflags;
 volatile sig_atomic_t	 mrtdump;
@@ -57,6 +59,7 @@ pid_t			 reconfpid;
 int			 reconfpending;
 struct imsgbuf		*ibuf_se;
 struct imsgbuf		*ibuf_rde;
+struct imsgbuf		*ibuf_rtr;
 struct rib_names	 ribnames = SIMPLEQ_HEAD_INITIALIZER(ribnames);
 char			*cname;
 char			*rcname;
@@ -90,10 +93,11 @@ usage(void)
 }
 
 #define PFD_PIPE_SESSION	0
-#define PFD_PIPE_ROUTE		1
-#define PFD_SOCK_ROUTE		2
-#define PFD_SOCK_PFKEY		3
-#define POLL_MAX		4
+#define PFD_PIPE_RDE		1
+#define PFD_PIPE_RTR		2
+#define PFD_SOCK_ROUTE		3
+#define PFD_SOCK_PFKEY		4
+#define POLL_MAX		5
 #define MAX_TIMEOUT		3600
 
 int	 cmd_opts;
@@ -102,32 +106,32 @@ int
 main(int argc, char *argv[])
 {
 	struct bgpd_config	*conf;
+	enum bgpd_process	 proc = PROC_MAIN;
 	struct rde_rib		*rr;
 	struct peer		*p;
 	struct pollfd		 pfd[POLL_MAX];
 	time_t			 timeout;
-	pid_t			 se_pid = 0, rde_pid = 0, pid;
+	pid_t			 se_pid = 0, rde_pid = 0, rtr_pid = 0, pid;
 	char			*conffile;
 	char			*saved_argv0;
 	int			 debug = 0;
-	int			 rflag = 0, sflag = 0;
 	int			 rfd, keyfd;
 	int			 ch, status;
 	int			 pipe_m2s[2];
 	int			 pipe_m2r[2];
+	int			 pipe_m2roa[2];
 
 	conffile = CONFFILE;
-	bgpd_process = PROC_MAIN;
 
 	log_init(1, LOG_DAEMON);	/* log to stderr until daemonized */
-	log_procinit(log_procnames[bgpd_process]);
+	log_procinit(log_procnames[PROC_MAIN]);
 	log_setverbose(1);
 
 	saved_argv0 = argv[0];
 	if (saved_argv0 == NULL)
 		saved_argv0 = "bgpd";
 
-	while ((ch = getopt(argc, argv, "cdD:f:nRSv")) != -1) {
+	while ((ch = getopt(argc, argv, "cdD:f:nRSTv")) != -1) {
 		switch (ch) {
 		case 'c':
 			cmd_opts |= BGPD_OPT_FORCE_DEMOTE;
@@ -152,10 +156,13 @@ main(int argc, char *argv[])
 			cmd_opts |= BGPD_OPT_VERBOSE;
 			break;
 		case 'R':
-			rflag = 1;
+			proc = PROC_RDE;
 			break;
 		case 'S':
-			sflag = 1;
+			proc = PROC_SE;
+			break;
+		case 'T':
+			proc = PROC_RTR;
 			break;
 		default:
 			usage();
@@ -165,11 +172,11 @@ main(int argc, char *argv[])
 
 	argc -= optind;
 	argv += optind;
-	if (argc > 0 || (sflag && rflag))
+	if (argc > 0)
 		usage();
 
 	if (cmd_opts & BGPD_OPT_NOACTION) {
-		if ((conf = parse_config(conffile, NULL)) == NULL)
+		if ((conf = parse_config(conffile, NULL, NULL)) == NULL)
 			exit(1);
 
 		if (cmd_opts & BGPD_OPT_VERBOSE)
@@ -185,10 +192,19 @@ main(int argc, char *argv[])
 		exit(0);
 	}
 
-	if (rflag)
+	switch (proc) {
+	case PROC_MAIN:
+		break;
+	case PROC_RDE:
 		rde_main(debug, cmd_opts & BGPD_OPT_VERBOSE);
-	else if (sflag)
+		/* NOTREACHED */
+	case PROC_SE:
 		session_main(debug, cmd_opts & BGPD_OPT_VERBOSE);
+		/* NOTREACHED */
+	case PROC_RTR:
+		rtr_main(debug, cmd_opts & BGPD_OPT_VERBOSE);
+		/* NOTREACHED */
+	}
 
 	if (geteuid())
 		errx(1, "need root privileges");
@@ -196,7 +212,7 @@ main(int argc, char *argv[])
 	if (getpwnam(BGPD_USER) == NULL)
 		errx(1, "unknown user %s", BGPD_USER);
 
-	if ((conf = parse_config(conffile, NULL)) == NULL) {
+	if ((conf = parse_config(conffile, NULL, NULL)) == NULL) {
 		log_warnx("config file %s has errors", conffile);
 		exit(1);
 	}
@@ -214,12 +230,15 @@ main(int argc, char *argv[])
 
 	getsockpair(pipe_m2s);
 	getsockpair(pipe_m2r);
+	getsockpair(pipe_m2roa);
 
 	/* fork children */
 	rde_pid = start_child(PROC_RDE, saved_argv0, pipe_m2r[1], debug,
 	    cmd_opts & BGPD_OPT_VERBOSE);
 	se_pid = start_child(PROC_SE, saved_argv0, pipe_m2s[1], debug,
 	    cmd_opts & BGPD_OPT_VERBOSE);
+	rtr_pid = start_child(PROC_RTR, saved_argv0, pipe_m2roa[1], debug,
+	    cmd_opts & BGPD_OPT_VERBOSE);
 
 	signal(SIGTERM, sighdlr);
 	signal(SIGINT, sighdlr);
@@ -229,10 +248,12 @@ main(int argc, char *argv[])
 	signal(SIGPIPE, SIG_IGN);
 
 	if ((ibuf_se = malloc(sizeof(struct imsgbuf))) == NULL ||
-	    (ibuf_rde = malloc(sizeof(struct imsgbuf))) == NULL)
+	    (ibuf_rde = malloc(sizeof(struct imsgbuf))) == NULL ||
+	    (ibuf_rtr = malloc(sizeof(struct imsgbuf))) == NULL)
 		fatal(NULL);
 	imsg_init(ibuf_se, pipe_m2s[0]);
 	imsg_init(ibuf_rde, pipe_m2r[0]);
+	imsg_init(ibuf_rtr, pipe_m2roa[0]);
 	mrt_init(ibuf_rde, ibuf_se);
 	if (kr_init(&rfd) == -1)
 		quit = 1;
@@ -257,7 +278,7 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 		fatal("pledge");
 #endif
 
-	if (imsg_send_sockets(ibuf_se, ibuf_rde))
+	if (imsg_send_sockets(ibuf_se, ibuf_rde, ibuf_rtr))
 		fatal("could not establish imsg links");
 	/* control setup needs to happen late since it sends imsgs */
 	if (control_setup(conf) == -1)
@@ -279,7 +300,8 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 		pfd[PFD_SOCK_PFKEY].events = POLLIN;
 
 		set_pollfd(&pfd[PFD_PIPE_SESSION], ibuf_se);
-		set_pollfd(&pfd[PFD_PIPE_ROUTE], ibuf_rde);
+		set_pollfd(&pfd[PFD_PIPE_RDE], ibuf_rde);
+		set_pollfd(&pfd[PFD_PIPE_RTR], ibuf_rtr);
 
 		if (timeout < 0 || timeout > MAX_TIMEOUT)
 			timeout = MAX_TIMEOUT;
@@ -301,15 +323,25 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 				quit = 1;
 		}
 
-		if (handle_pollfd(&pfd[PFD_PIPE_ROUTE], ibuf_rde) == -1) {
+		if (handle_pollfd(&pfd[PFD_PIPE_RDE], ibuf_rde) == -1) {
 			log_warnx("main: Lost connection to RDE");
 			msgbuf_clear(&ibuf_rde->w);
 			free(ibuf_rde);
 			ibuf_rde = NULL;
 			quit = 1;
 		} else {
-			if (dispatch_imsg(ibuf_rde, PFD_PIPE_ROUTE, conf) ==
-			    -1)
+			if (dispatch_imsg(ibuf_rde, PFD_PIPE_RDE, conf) == -1)
+				quit = 1;
+		}
+
+		if (handle_pollfd(&pfd[PFD_PIPE_RTR], ibuf_rtr) == -1) {
+			log_warnx("main: Lost connection to RTR");
+			msgbuf_clear(&ibuf_rtr->w);
+			free(ibuf_rtr);
+			ibuf_rtr = NULL;
+			quit = 1;
+		} else {
+			if (dispatch_imsg(ibuf_rtr, PFD_PIPE_RTR, conf) == -1)
 				quit = 1;
 		}
 
@@ -372,6 +404,12 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 		free(ibuf_rde);
 		ibuf_rde = NULL;
 	}
+	if (ibuf_rtr) {
+		msgbuf_clear(&ibuf_rtr->w);
+		close(ibuf_rtr->fd);
+		free(ibuf_rtr);
+		ibuf_rtr = NULL;
+	}
 
 	/* cleanup kernel data structures */
 	carp_demote_shutdown();
@@ -399,6 +437,8 @@ BROKEN	if (pledge("stdio rpath wpath cpath fattr unix route recvfd sendfd",
 				name = "route decision engine";
 			else if (pid == se_pid)
 				name = "session engine";
+			else if (pid == rtr_pid)
+				name = "rtr engine";
 			log_warnx("%s terminated; signal %d", name,
 				WTERMSIG(status));
 		}
@@ -444,6 +484,9 @@ start_child(enum bgpd_process p, char *argv0, int fd, int debug, int verbose)
 	case PROC_SE:
 		argv[argc++] = "-S";
 		break;
+	case PROC_RTR:
+		argv[argc++] = "-T";
+		break;
 	}
 	if (debug)
 		argv[argc++] = "-d";
@@ -476,7 +519,8 @@ reconfigure(char *conffile, struct bgpd_config *conf)
 		return (2);
 
 	log_info("rereading config");
-	if ((new_conf = parse_config(conffile, &conf->peers)) == NULL)
+	if ((new_conf = parse_config(conffile, &conf->peers,
+	    &conf->rtrs)) == NULL)
 		return (1);
 
 	merge_config(conf, new_conf);
@@ -503,8 +547,10 @@ send_config(struct bgpd_config *conf)
 	struct as_set		*aset;
 	struct prefixset	*ps;
 	struct prefixset_item	*psi, *npsi;
+	struct roa		*roa, *nroa;
+	struct rtr_config	*rtr;
 
-	reconfpending = 2;	/* one per child */
+	reconfpending = 3;	/* one per child */
 
 	expand_networks(conf);
 
@@ -517,6 +563,9 @@ send_config(struct bgpd_config *conf)
 	if (imsg_compose(ibuf_rde, IMSG_RECONF_CONF, 0, 0, -1,
 	    conf, sizeof(*conf)) == -1)
 		return (-1);
+	if (imsg_compose(ibuf_rtr, IMSG_RECONF_CONF, 0, 0, -1,
+	    conf, sizeof(*conf)) == -1)
+		return (-1);
 
 	TAILQ_FOREACH(la, conf->listen_addrs, entry) {
 		if (imsg_compose(ibuf_se, IMSG_RECONF_LISTENER, 0, 0, la->fd,
@@ -568,7 +617,6 @@ send_config(struct bgpd_config *conf)
 			if (imsg_compose(ibuf_rde, IMSG_RECONF_PREFIX_SET_ITEM,
 			    0, 0, -1, psi, sizeof(*psi)) == -1)
 				return (-1);
-			set_free(psi->set);
 			free(psi);
 		}
 		free(ps);
@@ -580,49 +628,28 @@ send_config(struct bgpd_config *conf)
 		if (imsg_compose(ibuf_rde, IMSG_RECONF_ORIGIN_SET, 0, 0, -1,
 		    ps->name, sizeof(ps->name)) == -1)
 			return (-1);
-		RB_FOREACH_SAFE(psi, prefixset_tree, &ps->psitems, npsi) {
-			struct roa_set *rs;
-			size_t i, l, n;
-			RB_REMOVE(prefixset_tree, &ps->psitems, psi);
-			rs = set_get(psi->set, &n);
-			for (i = 0; i < n; i += l) {
-				l = (n - i > 1024 ? 1024 : n - i);
-				if (imsg_compose(ibuf_rde,
-				    IMSG_RECONF_ROA_SET_ITEMS,
-				    0, 0, -1, rs + i, l * sizeof(*rs)) == -1)
-					return -1;
-			}
-			if (imsg_compose(ibuf_rde, IMSG_RECONF_PREFIX_SET_ITEM,
-			    0, 0, -1, psi, sizeof(*psi)) == -1)
+		RB_FOREACH_SAFE(roa, roa_tree, &ps->roaitems, nroa) {
+			RB_REMOVE(roa_tree, &ps->roaitems, roa);
+			if (imsg_compose(ibuf_rde, IMSG_RECONF_ROA_ITEM, 0, 0,
+			    -1, roa, sizeof(*roa)) == -1)
 				return (-1);
-			set_free(psi->set);
-			free(psi);
+			free(roa);
 		}
 		free(ps);
 	}
 
-	if (!RB_EMPTY(&conf->roa)) {
-		if (imsg_compose(ibuf_rde, IMSG_RECONF_ROA_SET, 0, 0, -1,
-		    NULL, 0) == -1)
+	/* roa table and rtr config are sent to the RTR engine */
+	RB_FOREACH_SAFE(roa, roa_tree, &conf->roa, nroa) {
+		RB_REMOVE(roa_tree, &conf->roa, roa);
+		if (imsg_compose(ibuf_rtr, IMSG_RECONF_ROA_ITEM, 0, 0,
+		    -1, roa, sizeof(*roa)) == -1)
+			return (-1);
+		free(roa);
+	}
+	SIMPLEQ_FOREACH(rtr, &conf->rtrs, entry) {
+		if (imsg_compose(ibuf_rtr, IMSG_RECONF_RTR_CONFIG, rtr->id,
+		    0, -1, rtr->descr, sizeof(rtr->descr)) == -1)
 			return (-1);
-		RB_FOREACH_SAFE(psi, prefixset_tree, &conf->roa, npsi) {
-			struct roa_set *rs;
-			size_t i, l, n;
-			RB_REMOVE(prefixset_tree, &conf->roa, psi);
-			rs = set_get(psi->set, &n);
-			for (i = 0; i < n; i += l) {
-				l = (n - i > 1024 ? 1024 : n - i);
-				if (imsg_compose(ibuf_rde,
-				    IMSG_RECONF_ROA_SET_ITEMS,
-				    0, 0, -1, rs + i, l * sizeof(*rs)) == -1)
-					return -1;
-			}
-			if (imsg_compose(ibuf_rde, IMSG_RECONF_PREFIX_SET_ITEM,
-			    0, 0, -1, psi, sizeof(*psi)) == -1)
-				return (-1);
-			set_free(psi->set);
-			free(psi);
-		}
 	}
 
 	/* as-sets for filters in the RDE */
@@ -712,6 +739,8 @@ send_config(struct bgpd_config *conf)
 		return (-1);
 	if (imsg_compose(ibuf_rde, IMSG_RECONF_DRAIN, 0, 0, -1, NULL, 0) == -1)
 		return (-1);
+	if (imsg_compose(ibuf_rtr, IMSG_RECONF_DRAIN, 0, 0, -1, NULL, 0) == -1)
+		return (-1);
 
 	/* mrt changes can be sent out of bound */
 	mrt_reconfigure(conf->mrt);
@@ -723,6 +752,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 {
 	struct imsg		 imsg;
 	struct peer		*p;
+	struct rtr_config	*r;
 	ssize_t			 n;
 	int			 rv, verbose;
 
@@ -736,7 +766,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 
 		switch (imsg.hdr.type) {
 		case IMSG_KROUTE_CHANGE:
-			if (idx != PFD_PIPE_ROUTE)
+			if (idx != PFD_PIPE_RDE)
 				log_warnx("route request not from RDE");
 			else if (imsg.hdr.len != IMSG_HEADER_SIZE +
 			    sizeof(struct kroute_full))
@@ -746,7 +776,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 				rv = -1;
 			break;
 		case IMSG_KROUTE_DELETE:
-			if (idx != PFD_PIPE_ROUTE)
+			if (idx != PFD_PIPE_RDE)
 				log_warnx("route request not from RDE");
 			else if (imsg.hdr.len != IMSG_HEADER_SIZE +
 			    sizeof(struct kroute_full))
@@ -756,7 +786,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 				rv = -1;
 			break;
 		case IMSG_KROUTE_FLUSH:
-			if (idx != PFD_PIPE_ROUTE)
+			if (idx != PFD_PIPE_RDE)
 				log_warnx("route request not from RDE");
 			else if (imsg.hdr.len != IMSG_HEADER_SIZE)
 				log_warnx("wrong imsg len");
@@ -764,7 +794,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 				rv = -1;
 			break;
 		case IMSG_NEXTHOP_ADD:
-			if (idx != PFD_PIPE_ROUTE)
+			if (idx != PFD_PIPE_RDE)
 				log_warnx("nexthop request not from RDE");
 			else if (imsg.hdr.len != IMSG_HEADER_SIZE +
 			    sizeof(struct bgpd_addr))
@@ -774,7 +804,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 				rv = -1;
 			break;
 		case IMSG_NEXTHOP_REMOVE:
-			if (idx != PFD_PIPE_ROUTE)
+			if (idx != PFD_PIPE_RDE)
 				log_warnx("nexthop request not from RDE");
 			else if (imsg.hdr.len != IMSG_HEADER_SIZE +
 			    sizeof(struct bgpd_addr))
@@ -784,7 +814,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 				    conf);
 			break;
 		case IMSG_PFTABLE_ADD:
-			if (idx != PFD_PIPE_ROUTE)
+			if (idx != PFD_PIPE_RDE)
 				log_warnx("pftable request not from RDE");
 			else
 				if (imsg.hdr.len != IMSG_HEADER_SIZE +
@@ -794,7 +824,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 					rv = -1;
 			break;
 		case IMSG_PFTABLE_REMOVE:
-			if (idx != PFD_PIPE_ROUTE)
+			if (idx != PFD_PIPE_RDE)
 				log_warnx("pftable request not from RDE");
 			else
 				if (imsg.hdr.len != IMSG_HEADER_SIZE +
@@ -804,7 +834,7 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 					rv = -1;
 			break;
 		case IMSG_PFTABLE_COMMIT:
-			if (idx != PFD_PIPE_ROUTE)
+			if (idx != PFD_PIPE_RDE)
 				log_warnx("pftable request not from RDE");
 			else if (imsg.hdr.len != IMSG_HEADER_SIZE)
 				log_warnx("wrong imsg len");
@@ -891,6 +921,9 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 				break;
 			}
 			if (idx == PFD_PIPE_SESSION) {
+				imsg_compose(ibuf_rtr, IMSG_RECONF_DONE, 0,
+				    0, -1, NULL, 0);
+			} else if (idx == PFD_PIPE_RTR) {
 				imsg_compose(ibuf_rde, IMSG_RECONF_DONE, 0,
 				    0, -1, NULL, 0);
 
@@ -915,9 +948,66 @@ dispatch_imsg(struct imsgbuf *ibuf, int idx, struct bgpd_config *conf)
 				 */
 				imsg_compose(ibuf_se, IMSG_RECONF_DONE, 0,
 				    0, -1, NULL, 0);
-				reconfpending = 2; /* expecting 2 DONE msg */
+				reconfpending = 3; /* expecting 2 DONE msg */
 			}
 			break;
+		case IMSG_SOCKET_CONN:
+			if (idx != PFD_PIPE_RTR) {
+				log_warnx("connect request not from RTR");
+			} else {
+				SIMPLEQ_FOREACH(r, &conf->rtrs, entry) {
+					if (imsg.hdr.peerid == r->id)
+						break;
+				}
+				if (r == NULL)
+					log_warnx("unknown rtr id %d",
+					    imsg.hdr.peerid);
+				else
+					bgpd_rtr_connect(r);
+			}
+			break;
+		case IMSG_CTL_SHOW_RTR:
+			if (idx == PFD_PIPE_SESSION) {
+				SIMPLEQ_FOREACH(r, &conf->rtrs, entry) {
+					imsg_compose(ibuf_rtr, imsg.hdr.type,
+					    r->id, imsg.hdr.pid, -1, NULL, 0);
+				}
+				imsg_compose(ibuf_rtr, IMSG_CTL_END,
+				    0, imsg.hdr.pid, -1, NULL, 0);
+			} else if (imsg.hdr.len != IMSG_HEADER_SIZE +
+			    sizeof(struct ctl_show_rtr)) {
+				log_warnx("IMSG_CTL_SHOW_RTR with wrong len");
+			} else if (idx == PFD_PIPE_RTR) {
+				SIMPLEQ_FOREACH(r, &conf->rtrs, entry) {
+					if (imsg.hdr.peerid == r->id)
+						break;
+				}
+				if (r != NULL) {
+					struct ctl_show_rtr *msg;
+					msg = imsg.data;
+					strlcpy(msg->descr, r->descr,
+					    sizeof(msg->descr));
+					msg->local_addr = r->local_addr;
+					msg->remote_addr = r->remote_addr;
+					msg->remote_port = r->remote_port;
+
+					imsg_compose(ibuf_se, imsg.hdr.type,
+					    imsg.hdr.peerid, imsg.hdr.pid,
+					    -1, imsg.data,
+					    imsg.hdr.len - IMSG_HEADER_SIZE);
+				}
+			}
+			break;
+		case IMSG_CTL_END:
+		case IMSG_CTL_SHOW_TIMER:
+			if (idx != PFD_PIPE_RTR) {
+				log_warnx("connect request not from RTR");
+				break;
+			}
+			imsg_compose(ibuf_se, imsg.hdr.type, imsg.hdr.peerid,
+			    imsg.hdr.pid, -1, imsg.data,
+			    imsg.hdr.len - IMSG_HEADER_SIZE);
+			break;
 		default:
 			break;
 		}
@@ -1134,13 +1224,15 @@ getsockpair(int pipe[2])
 }
 
 int
-imsg_send_sockets(struct imsgbuf *se, struct imsgbuf *rde)
+imsg_send_sockets(struct imsgbuf *se, struct imsgbuf *rde, struct imsgbuf *roa)
 {
 	int pipe_s2r[2];
 	int pipe_s2r_ctl[2];
+	int pipe_r2r[2];
 
 	getsockpair(pipe_s2r);
 	getsockpair(pipe_s2r_ctl);
+	getsockpair(pipe_r2r);
 
 	if (imsg_compose(se, IMSG_SOCKET_CONN, 0, 0, pipe_s2r[0],
 	    NULL, 0) == -1)
@@ -1156,5 +1248,44 @@ imsg_send_sockets(struct imsgbuf *se, struct imsgbuf *rde)
 	    NULL, 0) == -1)
 		return (-1);
 
+	if (imsg_compose(roa, IMSG_SOCKET_CONN_RTR, 0, 0, pipe_r2r[0],
+	    NULL, 0) == -1)
+		return (-1);
+	if (imsg_compose(rde, IMSG_SOCKET_CONN_RTR, 0, 0, pipe_r2r[1],
+	    NULL, 0) == -1)
+		return (-1);
+
 	return (0);
 }
+
+void
+bgpd_rtr_connect(struct rtr_config *r)
+{
+	socklen_t len;
+	int fd;
+
+	/* XXX should be non-blocking */
+	fd = socket(aid2af(r->remote_addr.aid), SOCK_STREAM, 0);
+	if (fd == -1) {
+		log_warn("rtr %s", r->descr);
+		return;
+	}
+	if (r->local_addr.aid != AID_UNSPEC) {
+		if (bind(fd,  addr2sa(&r->local_addr, 0, &len), len) == -1) {
+			log_warn("rtr %s: bind to %s", r->descr,
+			    log_addr(&r->local_addr));
+			close(fd);
+			return;
+		}
+	}
+
+	if (connect(fd, addr2sa(&r->remote_addr, r->remote_port, &len), len) ==
+	    -1) {
+		log_warn("rtr %s: connect to %s:%u", r->descr,
+		    log_addr(&r->remote_addr), r->remote_port);
+		close(fd);
+		return;
+	}
+
+	imsg_compose(ibuf_rtr, IMSG_SOCKET_CONN, r->id, 0, fd, NULL, 0);
+}
