@@ -1,4 +1,4 @@
-/*	$OpenBSD: printconf.c,v 1.142 2020/04/23 16:13:11 claudio Exp $	*/
+/*	$OpenBSD: printconf.c,v 1.147 2021/03/02 09:45:07 claudio Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -41,7 +41,8 @@ void		 print_network(struct network_config *, const char *);
 void		 print_as_sets(struct as_set_head *);
 void		 print_prefixsets(struct prefixset_head *);
 void		 print_originsets(struct prefixset_head *);
-void		 print_roa(struct prefixset_tree *p);
+void		 print_roa(struct roa_tree *);
+void		 print_rtrs(struct rtr_config_head *);
 void		 print_peer(struct peer_config *, struct bgpd_config *,
 		    const char *);
 const char	*print_auth_alg(u_int8_t);
@@ -356,6 +357,7 @@ print_set(struct filter_set_head *set)
 			break;
 		case ACTION_RTLABEL_ID:
 		case ACTION_PFTABLE_ID:
+		case ACTION_SET_NEXTHOP_REF:
 			/* not possible */
 			printf("king bula saiz: config broken");
 			break;
@@ -388,9 +390,13 @@ print_mainconf(struct bgpd_config *conf)
 
 	if (conf->flags & BGPD_FLAG_DECISION_ROUTEAGE)
 		printf("rde route-age evaluate\n");
-
 	if (conf->flags & BGPD_FLAG_DECISION_MED_ALWAYS)
 		printf("rde med compare always\n");
+	if (conf->flags & BGPD_FLAG_DECISION_ALL_PATHS)
+		printf("rde evaluate all\n");
+
+	if (conf->flags & BGPD_FLAG_NO_AS_SET)
+		printf("reject as-set yes\n");
 
 	if (conf->log & BGPD_LOG_UPDATES)
 		printf("log updates\n");
@@ -534,50 +540,61 @@ void
 print_originsets(struct prefixset_head *psh)
 {
 	struct prefixset	*ps;
-	struct prefixset_item	*psi;
-	struct roa_set		*rs;
-	size_t			 i, n;
+	struct roa		*roa;
+	struct bgpd_addr	 addr;
 
 	SIMPLEQ_FOREACH(ps, psh, entry) {
 		printf("origin-set \"%s\" {", ps->name);
-		RB_FOREACH(psi, prefixset_tree, &ps->psitems) {
-			rs = set_get(psi->set, &n);
-			for (i = 0; i < n; i++) {
-				printf("\n\t");
-				print_prefix(&psi->p);
-				if (psi->p.len != rs[i].maxlen)
-					printf(" maxlen %u", rs[i].maxlen);
-				printf(" source-as %u", rs[i].as);
-			}
+		RB_FOREACH(roa, roa_tree, &ps->roaitems) {
+			printf("\n\t");
+			addr.aid = roa->aid;
+			addr.v6 = roa->prefix.inet6;
+			printf("%s/%u", log_addr(&addr), roa->prefixlen);
+			if (roa->prefixlen != roa->maxlen)
+				printf(" maxlen %u", roa->maxlen);
+			printf(" source-as %u", roa->asnum);
 		}
 		printf("\n}\n\n");
 	}
 }
 
 void
-print_roa(struct prefixset_tree *p)
+print_roa(struct roa_tree *r)
 {
-	struct prefixset_item	*psi;
-	struct roa_set		*rs;
-	size_t			 i, n;
+	struct roa	*roa;
+	struct bgpd_addr addr;
 
-	if (RB_EMPTY(p))
+	if (RB_EMPTY(r))
 		return;
 
 	printf("roa-set {");
-	RB_FOREACH(psi, prefixset_tree, p) {
-		rs = set_get(psi->set, &n);
-		for (i = 0; i < n; i++) {
-			printf("\n\t");
-			print_prefix(&psi->p);
-			if (psi->p.len != rs[i].maxlen)
-				printf(" maxlen %u", rs[i].maxlen);
-			printf(" source-as %u", rs[i].as);
-		}
+	RB_FOREACH(roa, roa_tree, r) {
+		printf("\n\t");
+		addr.aid = roa->aid;
+		addr.v6 = roa->prefix.inet6;
+		printf("%s/%u", log_addr(&addr), roa->prefixlen);
+		if (roa->prefixlen != roa->maxlen)
+			printf(" maxlen %u", roa->maxlen);
+		printf(" source-as %u", roa->asnum);
 	}
 	printf("\n}\n\n");
 }
 
+void
+print_rtrs(struct rtr_config_head *rh)
+{
+	struct rtr_config *r;
+
+	SIMPLEQ_FOREACH(r, rh, entry) {
+		printf("rtr %s {\n", log_addr(&r->remote_addr));
+		printf("\tdescr \"%s\"\n", r->descr);
+		printf("\tport %u\n", r->remote_port);
+		if (r->local_addr.aid != AID_UNSPEC)
+			printf("local-addr %s\n", log_addr(&r->local_addr));
+		printf("}\n\n");
+	}
+}
+
 void
 print_peer(struct peer_config *p, struct bgpd_config *conf, const char *c)
 {
@@ -666,6 +683,22 @@ print_peer(struct peer_config *p, struct bgpd_config *conf, const char *c)
 	if (p->flags & PEERFLAG_TRANS_AS)
 		printf("%s\ttransparent-as yes\n", c);
 
+	if (conf->flags & BGPD_FLAG_DECISION_ALL_PATHS) {
+		if (!(p->flags & PEERFLAG_EVALUATE_ALL))
+			printf("%s\trde evaluate default\n", c);
+	} else {
+		if (p->flags & PEERFLAG_EVALUATE_ALL)
+			printf("%s\trde evaluate all\n", c);
+	}
+
+	if (conf->flags & BGPD_FLAG_NO_AS_SET) {
+		if (!(p->flags & PEERFLAG_NO_AS_SET))
+			printf("%s\treject as-set no\n", c);
+	} else {
+		if (p->flags & PEERFLAG_NO_AS_SET)
+			printf("%s\treject as-set yes\n", c);
+	}
+
 	if (p->flags & PEERFLAG_LOG_UPDATES)
 		printf("%s\tlog updates\n", c);
 
@@ -1002,6 +1035,7 @@ print_config(struct bgpd_config *conf, struct rib_names *rib_l)
 	struct l3vpn		*vpn;
 
 	print_mainconf(conf);
+	print_rtrs(&conf->rtrs);
 	print_roa(&conf->roa);
 	print_as_sets(&conf->as_sets);
 	print_prefixsets(&conf->prefixsets);
